#!/cs/local/bin/perl
# transform from bpseq format to dot-parentheses format

use strict;

if ($#ARGV != 1)
  {
    print "Usage: $0 <input_bpseq_file> <output_dp_file>\n";
    exit;
  }

my $bpseq_file = $ARGV[0];
my $dp_file = $ARGV[1];

my $analyser = "./Analyser -analyse";
my @res;
my $line;
my @seq = ();
my @pair = ();
my @str = ();
#my @arcs2remove = ();
my @bands2remove = ();
my $item;
my $header = "";
#my @left_of_arc;
#my @right_of_arc;
my $command;
my $i;

my @pk_left  = ('[','{','<','A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P','Q','R','S','T','U','V','W','X','Y','Z');
my @pk_right = (']','}','>','a','b','c','d','e','f','g','h','i','j','k','l','m','n','o','p','q','r','s','t','u','v','w','x','y','z');



open (BPSEQ, $bpseq_file) || die ("Cannot open $bpseq_file");
while ($line = <BPSEQ>)
  {
    if ($line =~ /^[#\n]/) { $header .= $line; }
    elsif ($line =~ /(\d+)\s+(\S+)\s+(\d+)/)
      {
	#print "$1 $2 $3\n";
	$seq[$1] = $2;
	$pair[$1] = $3;
	if ($3 == 0) { $str[$1] = "."; }
	elsif ($3 > $1) { $str[$1] = "("; }
	else { $str[$1] = ")"; }
      }
    else
      { print "Bad line: $line\n"; exit; }
  }
close (BPSEQ);

$bpseq_file =~ s/\.bpseq//;

# first create the arc array

#@left_of_arc = ();
#@right_of_arc = ();
#for ($i=1; $i <= $#seq; $i++)
#  {
#    if ($pair[$i] > $i)
#      {
#	push (@left_of_arc, $i);
#	push (@right_of_arc, $pair[$i]);
#      }
#  }

#print "Left:\n@left_of_arc\n";
#print "Right:\n@right_of_arc\n";

# get both the arcs to be removed and the bands
# we need the number of bands to be removed so that we know how many letters we need
# we need the arcs because the bands may have free bases

$command = "$analyser $bpseq_file.bpseq | grep \"Coordinates of the bands to be removed\" -C 1 -B 0";
#print "Calling: $command\n";
@res = `$command`;
@bands2remove = split (/;/, $res[1]);

#print "@bands2remove\n";

my $letter=0;  # letter to be used for the pseudoknotted pairs

foreach $item (@bands2remove)     # 16-57, 18-54
  {
    if ($item =~ /(\d+)\-(\d+), (\d+)\-(\d+)/)
      {
	my $begin1 = $1;
	my $end1 = $3;
	my $begin2 = $4;
	my $end2 = $2;

	for (my $k = $begin1; $k <= $end1; $k++)
	  {
	    if ($pair[$k] > 0)
	      {
		$str[$k] = $pk_left[$letter];
	      }
	  }
	for (my $k = $begin2; $k <= $end2; $k++)
	  {
	    if ($pair[$k] > 0)
	      {
		$str[$k] = $pk_right[$letter];
	      }
	  }
	$letter++;
      }
  }


# now print the dp file
open (DP, ">$dp_file") || die ("Cannot open $dp_file");

# print the sequence and structure in the output file

#$header =~ s/bpseq/dp/;
#print DP "$header\n";
# for (my $i = 1; $i <= $#seq; $i++)
#   {
#     print DP $seq[$i];
#     if ($i % 50 == 0) { print DP "\n"; }
#   }
# print DP "\n";
for (my $i = 1; $i <= $#str; $i++)
  {
    print DP "$str[$i]";
  #  if ($i % 50 == 0) { print DP "\n"; }
  }
print DP "\n";

close (DP);


