use strict;

# after the prediction and analysis step was run via SGE, now we have to average the results and add the average to the results file

# there might be more sequences per res file
#### assume there's only one sequence per res file!!!! NO, there are many sequences per file. For each sequence, there is a line in the results file.
# Fixed that on Apr 18, 2008

if ($#ARGV != 0)
{
    print "Usage: $0 <input_file>\n";
    # header is "Training set" or "Validation set", to be written in the results file
    exit;
}

my $file = $ARGV[0];
print "$file\n";

my $resfile;
my $line;

my @initial_sens = ();
my @initial_ppv = ();
my @initial_f = ();	# added this on Feb 8, 2009, to compute f for every structure 


my @new_sens = ();
my @new_ppv = ();
my @new_f = ();

my $initial_avg_sens;
my $initial_avg_ppv;
my $initial_avg_f;

my $new_avg_sens;
my $new_avg_ppv;
my $new_avg_f;

my $initial_std_sens;
my $initial_std_ppv;
my $initial_std_f;

my $new_std_sens;
my $new_std_ppv;
my $new_std_f;

my $i;

if (-z $file)
{
    print "ERROR!, File $file is empty! An error must have happened!\n";
    exit;
}

open (FILE, $file) || die ("Cannot open $file");
while ($line = <FILE>)
{

    # Initial set sens: 0.697, Initial set ppv: 0.622, New set sens: 0.545, New set ppv: 0.692
    if ($line =~ /Initial set sens: (\S+), Initial set ppv: (\S+), New set sens: (\S+), New set ppv: (\S+)/)
    {
        ($1 >= 0) or die("initial sens $1 should be >= 0!");
        push (@initial_sens, $1);   
        ($2 >= 0) or die("initial ppv $2 should be >= 0!");
        push (@initial_ppv, $2);   
        push (@initial_f, fmeasure($1,$2));
        ($3 >= 0) or die("new sens $3 should be >= 0!");
        push (@new_sens, $3);   
        ($4 >= 0) or die("new ppv $4 should be >= 0!");
        push (@new_ppv, $4);    
        push (@new_f, fmeasure($3,$4)); 
    }
}
close (FILE);

($initial_avg_sens, $initial_std_sens) = get_avg_and_std (@initial_sens);
($initial_avg_ppv, $initial_std_ppv) = get_avg_and_std (@initial_ppv);
($initial_avg_f, $initial_std_f) = get_avg_and_std (@initial_f);


($new_avg_sens, $new_std_sens) = get_avg_and_std (@new_sens);
($new_avg_ppv, $new_std_ppv) = get_avg_and_std (@new_ppv);
($new_avg_f, $new_std_f) = get_avg_and_std (@new_f);

# Now I'm computing f for every structure and then getting the average

my $initial_f = 0;
my $new_f = 0;
if ($initial_avg_sens+$initial_avg_ppv != 0)
{
    $initial_f = 2* $initial_avg_sens * $initial_avg_ppv/($initial_avg_sens+$initial_avg_ppv);
}
if ($new_avg_sens+$new_avg_ppv != 0)
{
    $new_f = 2* $new_avg_sens * $new_avg_ppv/($new_avg_sens+$new_avg_ppv);
}

print "------\n";

printf ("Initial: avg_sens: %.3f \\pm %.3f, avg_ppv: %.3f \\pm %.3f, f-measure: %.3f\n", $initial_avg_sens, $initial_std_sens, $initial_avg_ppv, $initial_std_ppv, $initial_avg_f);

printf ("Newpar : avg_sens: %.3f \\pm %.3f, avg_ppv: %.3f \\pm %.3f, f-measure: %.3f\n", $new_avg_sens, $new_std_sens, $new_avg_ppv, $new_std_ppv, $new_avg_f);

##############################

sub fmeasure 
{
    my ($sn, $ppv) = @_;
    if ($sn+$ppv == 0)  { return 0;  }
    return (2*$sn*$ppv)/($sn+$ppv);
}

##############################

sub get_avg_and_std
{
    # compute the avg and std of all the elements of this array
    my (@array) = @_;
    my $avg;
    my $std;
    my $num = @array."";
    my $sum = 0;
    my $sum2 = 0;
    my $a;
    foreach $a (@array)
    {
        $sum += $a;
        $sum2 += $a*$a;
    }
    # assume $num > 0 , maybe I should check
    $avg = $sum/$num;
    $std = sqrt ($sum2/$num - $avg*$avg);
    return ($avg, $std);
}




