use strict;

# parses the file that contains sequence, known structure, initial structure, many suboptimal predicted structures.

if ($#ARGV != 1)
{
    print "Usage: $0 <directory> <measure:sens|ppv|F-measure>\n";
    exit;
}

my $dir = $ARGV[0];
my $measure = $ARGV[1];

my $line;

my @initial_f = ();
my @new_f = ();
my $initial_avg_f;
my $initial_std_f;
my $new_avg_f;
my $new_std_f;
my $file;

foreach $file (glob ("$dir/out*.txt"))
{
    open (FILE, $file) || die ("Cannot open $file");
    while ($line = <FILE>)
    {
    
        # Initial sens: 0.773, Initial ppv: 0.823, Initial F-measure: 0.797
        if ($line =~ /Initial $measure: (\S+)/)
        {
            push (@initial_f, $1);
            #print "Initial: $1\t";
            # the next line has the accuracy of the Hotknots best prediction
            $line = <FILE>;
            # New DP  F-measure: 0.559
            if ($line =~ /New.*$measure: (\S+)/)
            {
                push (@new_f, $1);
                #print "New: $1\n";
            }
        }
    }
    close (FILE);
}

($initial_avg_f, $initial_std_f) = get_avg_and_std (@initial_f);
($new_avg_f, $new_std_f) = get_avg_and_std (@new_f);

printf ("Initial $measure: %.3f, New $measure: %.3f\n",  $initial_avg_f, $new_avg_f);

##############################

sub get_avg_and_std
{
    # compute the avg and std of all the elements of this array
    my (@array) = @_;
    my $avg;
    my $std;
    my $num = @array."";
    my $sum = 0;
    my $sum2 = 0;
    my $a;
    foreach $a (@array)
    {
        $sum += $a;
        $sum2 += $a*$a;
    }
    # assume $num > 0 , maybe I should check
    $avg = $sum/$num;
    $std = sqrt ($sum2/$num - $avg*$avg);
    return ($avg, $std);
}




